@@ -1,55 +1,68 @@
 package k8shandler
 
 import (
+	"context"
 	"fmt"
+	"reflect"
 	"time"
 
 	monitoringv1 "github.com/coreos/prometheus-operator/pkg/apis/monitoring/v1"
 	logging "github.com/openshift/cluster-logging-operator/pkg/apis/logging/v1"
+	logforward "github.com/openshift/cluster-logging-operator/pkg/apis/logging/v1alpha1"
+	"github.com/openshift/cluster-logging-operator/pkg/constants"
+	"github.com/openshift/cluster-logging-operator/pkg/logger"
 	"github.com/openshift/cluster-logging-operator/pkg/utils"
 	"github.com/sirupsen/logrus"
 	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/apimachinery/pkg/util/intstr"
 	"k8s.io/client-go/util/retry"
 
+	configv1 "github.com/openshift/api/config/v1"
 	apps "k8s.io/api/apps/v1"
 	v1 "k8s.io/api/core/v1"
 )
 
 const (
 	fluentdAlertsFile = "fluentd/fluentd_prometheus_alerts.yaml"
+	fluentdName       = "fluentd"
 )
 
 func (clusterRequest *ClusterLoggingRequest) removeFluentd() (err error) {
 	if clusterRequest.isManaged() {
 
-		if err = clusterRequest.RemoveService("fluentd"); err != nil {
+		if err = clusterRequest.RemoveService(fluentdName); err != nil {
 			return
 		}
 
-		if err = clusterRequest.RemoveServiceMonitor("fluentd"); err != nil {
+		if err = clusterRequest.RemoveServiceMonitor(fluentdName); err != nil {
 			return
 		}
 
-		if err = clusterRequest.RemovePrometheusRule("fluentd"); err != nil {
+		if err = clusterRequest.RemovePrometheusRule(fluentdName); err != nil {
 			return
 		}
 
-		if err = clusterRequest.RemoveConfigMap("fluentd"); err != nil {
+		if err = clusterRequest.RemoveConfigMap(fluentdName); err != nil {
 			return
 		}
 
-		if err = clusterRequest.RemoveDaemonset("fluentd"); err != nil {
+		if err = clusterRequest.RemoveConfigMap(constants.FluentdTrustedCAName); err != nil {
+			return
+		}
+
+		if err = clusterRequest.RemoveDaemonset(fluentdName); err != nil {
 			return
 		}
 
 		// Wait longer than the terminationGracePeriodSeconds
 		time.Sleep(12 * time.Second)
 
-		if err = clusterRequest.RemoveSecret("fluentd"); err != nil {
+		if err = clusterRequest.RemoveSecret(fluentdName); err != nil {
 			return
 		}
+
 	}
 
 	return nil
@@ -57,9 +70,9 @@ func (clusterRequest *ClusterLoggingRequest) removeFluentd() (err error) {
 
 func (clusterRequest *ClusterLoggingRequest) createOrUpdateFluentdService() error {
 	service := NewService(
-		"fluentd",
+		fluentdName,
 		clusterRequest.cluster.Namespace,
-		"fluentd",
+		fluentdName,
 		[]v1.ServicePort{
 			{
 				Port:       metricsPort,
@@ -87,7 +100,7 @@ func (clusterRequest *ClusterLoggingRequest) createOrUpdateFluentdServiceMonitor
 
 	cluster := clusterRequest.cluster
 
-	serviceMonitor := NewServiceMonitor("fluentd", cluster.Namespace)
+	serviceMonitor := NewServiceMonitor(fluentdName, cluster.Namespace)
 
 	endpoint := monitoringv1.Endpoint{
 		Port:   metricsPortName,
@@ -95,7 +108,7 @@ func (clusterRequest *ClusterLoggingRequest) createOrUpdateFluentdServiceMonitor
 		Scheme: "https",
 		TLSConfig: &monitoringv1.TLSConfig{
 			CAFile:     prometheusCAFile,
-			ServerName: fmt.Sprintf("%s.%s.svc", "fluentd", cluster.Namespace),
+			ServerName: fmt.Sprintf("%s.%s.svc", fluentdName, cluster.Namespace),
 			// ServerName can be e.g. fluentd.openshift-logging.svc
 		},
 	}
@@ -129,7 +142,7 @@ func (clusterRequest *ClusterLoggingRequest) createOrUpdateFluentdPrometheusRule
 
 	cluster := clusterRequest.cluster
 
-	promRule := NewPrometheusRule("fluentd", cluster.Namespace)
+	promRule := NewPrometheusRule(fluentdName, cluster.Namespace)
 
 	promRuleSpec, err := NewPrometheusRuleSpecFrom(utils.GetShareDir() + "/" + fluentdAlertsFile)
 	if err != nil {
@@ -148,15 +161,30 @@ func (clusterRequest *ClusterLoggingRequest) createOrUpdateFluentdPrometheusRule
 	return nil
 }
 
-func (clusterRequest *ClusterLoggingRequest) createOrUpdateFluentdConfigMap() error {
+//createOrUpdateFluentdLegacyForwardConfigMap to address Bug 1782566. To be removed for LogForwarding GA
+func (clusterRequest *ClusterLoggingRequest) includeLegacyForwardConfig() bool {
+	config := &v1.ConfigMap{
+		Data: map[string]string{},
+	}
+	if err := clusterRequest.Get("secure-forward", config); err != nil {
+		if errors.IsNotFound(err) {
+			return false
+		}
+		logger.Warnf("There was a non-critical error trying to fetch the secure-forward configmap: %v", err)
+	}
+	_, found := config.Data["secure-forward.conf"]
+	return found
+}
 
+func (clusterRequest *ClusterLoggingRequest) createOrUpdateFluentdConfigMap(fluentConf string) error {
+	logrus.Debug("createOrUpdateFluentdConfigMap...")
 	fluentdConfigMap := NewConfigMap(
-		"fluentd",
+		fluentdName,
 		clusterRequest.cluster.Namespace,
 		map[string]string{
-			"fluent.conf":          string(utils.GetFileContents(utils.GetShareDir() + "/fluentd/fluent.conf")),
+			"fluent.conf":          fluentConf,
 			"throttle-config.yaml": string(utils.GetFileContents(utils.GetShareDir() + "/fluentd/fluentd-throttle-config.yaml")),
-			"secure-forward.conf":  string(utils.GetFileContents(utils.GetShareDir() + "/fluentd/secure-forward.conf")),
+			"run.sh":               string(utils.GetFileContents(utils.GetShareDir() + "/fluentd/run.sh")),
 		},
 	)
 
@@ -166,21 +194,34 @@ func (clusterRequest *ClusterLoggingRequest) createOrUpdateFluentdConfigMap() er
 	if err != nil && !errors.IsAlreadyExists(err) {
 		return fmt.Errorf("Failure constructing Fluentd configmap: %v", err)
 	}
+	retryErr := retry.RetryOnConflict(retry.DefaultRetry, func() error {
+		current := &v1.ConfigMap{}
+		if err = clusterRequest.Get(fluentdConfigMap.Name, current); err != nil {
+			if errors.IsNotFound(err) {
+				logrus.Debugf("Returning nil. The configmap %q was not found even though create previously failed.  Was it culled?", fluentdConfigMap.Name)
+				return nil
+			}
+			return fmt.Errorf("Failed to get %v configmap for %q: %v", fluentdConfigMap.Name, clusterRequest.cluster.Name, err)
+		}
+		if reflect.DeepEqual(fluentdConfigMap.Data, current.Data) {
+			return nil
+		}
+		current.Data = fluentdConfigMap.Data
+		return clusterRequest.Update(current)
+	})
 
-	return nil
+	return retryErr
 }
+
 func (clusterRequest *ClusterLoggingRequest) createOrUpdateFluentdSecret() error {
 
 	fluentdSecret := NewSecret(
-		"fluentd",
+		fluentdName,
 		clusterRequest.cluster.Namespace,
 		map[string][]byte{
-			"app-ca":     utils.GetWorkingDirFileContents("ca.crt"),
-			"app-key":    utils.GetWorkingDirFileContents("system.logging.fluentd.key"),
-			"app-cert":   utils.GetWorkingDirFileContents("system.logging.fluentd.crt"),
-			"infra-ca":   utils.GetWorkingDirFileContents("ca.crt"),
-			"infra-key":  utils.GetWorkingDirFileContents("system.logging.fluentd.key"),
-			"infra-cert": utils.GetWorkingDirFileContents("system.logging.fluentd.crt"),
+			"ca-bundle.crt": utils.GetWorkingDirFileContents("ca.crt"),
+			"tls.key":       utils.GetWorkingDirFileContents("system.logging.fluentd.key"),
+			"tls.crt":       utils.GetWorkingDirFileContents("system.logging.fluentd.crt"),
 		})
 
 	utils.AddOwnerRefToObject(fluentdSecret, utils.AsOwner(clusterRequest.cluster))
@@ -193,8 +234,12 @@ func (clusterRequest *ClusterLoggingRequest) createOrUpdateFluentdSecret() error
 	return nil
 }
 
-func newFluentdPodSpec(logging *logging.ClusterLogging, elasticsearchAppName string, elasticsearchInfraName string) v1.PodSpec {
-	var resources = logging.Spec.Collection.Logs.FluentdSpec.Resources
+func newFluentdPodSpec(cluster *logging.ClusterLogging, elasticsearchAppName string, elasticsearchInfraName string, proxyConfig *configv1.Proxy, trustedCABundleCM *v1.ConfigMap, pipelineSpec logforward.ForwardingSpec) v1.PodSpec {
+	collectionSpec := logging.CollectionSpec{}
+	if cluster.Spec.Collection != nil {
+		collectionSpec = *cluster.Spec.Collection
+	}
+	var resources = collectionSpec.Logs.FluentdSpec.Resources
 	if resources == nil {
 		resources = &v1.ResourceRequirements{
 			Limits: v1.ResourceList{v1.ResourceMemory: defaultFluentdMemory},
@@ -215,36 +260,32 @@ func newFluentdPodSpec(logging *logging.ClusterLogging, elasticsearchAppName str
 	}
 
 	fluentdContainer.Env = []v1.EnvVar{
-		{Name: "NODE_NAME", ValueFrom: &v1.EnvVarSource{FieldRef: &v1.ObjectFieldSelector{FieldPath: "spec.nodeName"}}},
+		{Name: "NODE_NAME", ValueFrom: &v1.EnvVarSource{FieldRef: &v1.ObjectFieldSelector{APIVersion: "v1", FieldPath: "spec.nodeName"}}},
 		{Name: "MERGE_JSON_LOG", Value: "false"},
 		{Name: "PRESERVE_JSON_LOG", Value: "true"},
 		{Name: "K8S_HOST_URL", Value: "https://kubernetes.default.svc"},
-		{Name: "ES_HOST", Value: elasticsearchAppName},
-		{Name: "ES_PORT", Value: "9200"},
-		{Name: "ES_CLIENT_CERT", Value: "/etc/fluent/keys/app-cert"},
-		{Name: "ES_CLIENT_KEY", Value: "/etc/fluent/keys/app-key"},
-		{Name: "ES_CA", Value: "/etc/fluent/keys/app-ca"},
 		{Name: "METRICS_CERT", Value: "/etc/fluent/metrics/tls.crt"},
 		{Name: "METRICS_KEY", Value: "/etc/fluent/metrics/tls.key"},
-		{Name: "OPS_HOST", Value: elasticsearchInfraName},
-		{Name: "OPS_PORT", Value: "9200"},
-		{Name: "OPS_CLIENT_CERT", Value: "/etc/fluent/keys/infra-cert"},
-		{Name: "OPS_CLIENT_KEY", Value: "/etc/fluent/keys/infra-key"},
-		{Name: "OPS_CA", Value: "/etc/fluent/keys/infra-ca"},
 		{Name: "BUFFER_QUEUE_LIMIT", Value: "32"},
 		{Name: "BUFFER_SIZE_LIMIT", Value: "8m"},
 		{Name: "FILE_BUFFER_LIMIT", Value: "256Mi"},
 		{Name: "FLUENTD_CPU_LIMIT", ValueFrom: &v1.EnvVarSource{ResourceFieldRef: &v1.ResourceFieldSelector{ContainerName: "fluentd", Resource: "limits.cpu"}}},
 		{Name: "FLUENTD_MEMORY_LIMIT", ValueFrom: &v1.EnvVarSource{ResourceFieldRef: &v1.ResourceFieldSelector{ContainerName: "fluentd", Resource: "limits.memory"}}},
-		{Name: "NODE_IPV4", ValueFrom: &v1.EnvVarSource{FieldRef: &v1.ObjectFieldSelector{FieldPath: "status.hostIP"}}},
+		{Name: "NODE_IPV4", ValueFrom: &v1.EnvVarSource{FieldRef: &v1.ObjectFieldSelector{APIVersion: "v1", FieldPath: "status.hostIP"}}},
 		{Name: "CDM_KEEP_EMPTY_FIELDS", Value: "message"}, // by default, keep empty messages
 	}
 
+	proxyEnv := utils.SetProxyEnvVars(proxyConfig)
+	fluentdContainer.Env = append(fluentdContainer.Env, proxyEnv...)
+
 	fluentdContainer.VolumeMounts = []v1.VolumeMount{
 		{Name: "runlogjournal", MountPath: "/run/log/journal"},
 		{Name: "varlog", MountPath: "/var/log"},
 		{Name: "varlibdockercontainers", ReadOnly: true, MountPath: "/var/lib/docker"},
 		{Name: "config", ReadOnly: true, MountPath: "/etc/fluent/configs.d/user"},
+		{Name: "secureforwardconfig", ReadOnly: true, MountPath: "/etc/fluent/configs.d/secure-forward"},
+		{Name: "secureforwardcerts", ReadOnly: true, MountPath: "/etc/ocp-forward"},
+		{Name: "entrypoint", ReadOnly: true, MountPath: "/opt/app-root/src/run.sh", SubPath: "run.sh"},
 		{Name: "certs", ReadOnly: true, MountPath: "/etc/fluent/keys"},
 		{Name: "localtime", ReadOnly: true, MountPath: "/etc/localtime"},
 		{Name: "dockercfg", ReadOnly: true, MountPath: "/etc/sysconfig/docker"},
@@ -252,13 +293,31 @@ func newFluentdPodSpec(logging *logging.ClusterLogging, elasticsearchAppName str
 		{Name: "filebufferstorage", MountPath: "/var/lib/fluentd"},
 		{Name: metricsVolumeName, MountPath: "/etc/fluent/metrics"},
 	}
+	for _, target := range pipelineSpec.Outputs {
+		if target.Secret != nil {
+			path := fmt.Sprintf("/var/run/ocp-collector/secrets/%s", target.Secret.Name)
+			fluentdContainer.VolumeMounts = append(fluentdContainer.VolumeMounts, v1.VolumeMount{Name: target.Name, MountPath: path})
+		}
+	}
+
+	addTrustedCAVolume := false
+	// If trusted CA bundle ConfigMap exists and its hash value is non-zero, mount the bundle.
+	if trustedCABundleCM != nil && hasTrustedCABundle(trustedCABundleCM) {
+		addTrustedCAVolume = true
+		fluentdContainer.VolumeMounts = append(fluentdContainer.VolumeMounts,
+			v1.VolumeMount{
+				Name:      constants.FluentdTrustedCAName,
+				ReadOnly:  true,
+				MountPath: constants.TrustedCABundleMountDir,
+			})
+	}
 
 	fluentdContainer.SecurityContext = &v1.SecurityContext{
 		Privileged: utils.GetBool(true),
 	}
 
 	tolerations := utils.AppendTolerations(
-		logging.Spec.Collection.Logs.FluentdSpec.Tolerations,
+		collectionSpec.Logs.FluentdSpec.Tolerations,
 		[]v1.Toleration{
 			v1.Toleration{
 				Key:      "node-role.kubernetes.io/master",
@@ -281,6 +340,9 @@ func newFluentdPodSpec(logging *logging.ClusterLogging, elasticsearchAppName str
 			{Name: "varlog", VolumeSource: v1.VolumeSource{HostPath: &v1.HostPathVolumeSource{Path: "/var/log"}}},
 			{Name: "varlibdockercontainers", VolumeSource: v1.VolumeSource{HostPath: &v1.HostPathVolumeSource{Path: "/var/lib/docker"}}},
 			{Name: "config", VolumeSource: v1.VolumeSource{ConfigMap: &v1.ConfigMapVolumeSource{LocalObjectReference: v1.LocalObjectReference{Name: "fluentd"}}}},
+			{Name: "secureforwardconfig", VolumeSource: v1.VolumeSource{ConfigMap: &v1.ConfigMapVolumeSource{LocalObjectReference: v1.LocalObjectReference{Name: "secure-forward"}, Optional: utils.GetBool(true)}}},
+			{Name: "secureforwardcerts", VolumeSource: v1.VolumeSource{Secret: &v1.SecretVolumeSource{SecretName: "secure-forward", Optional: utils.GetBool(true)}}},
+			{Name: "entrypoint", VolumeSource: v1.VolumeSource{ConfigMap: &v1.ConfigMapVolumeSource{LocalObjectReference: v1.LocalObjectReference{Name: "fluentd"}}}},
 			{Name: "certs", VolumeSource: v1.VolumeSource{Secret: &v1.SecretVolumeSource{SecretName: "fluentd"}}},
 			{Name: "localtime", VolumeSource: v1.VolumeSource{HostPath: &v1.HostPathVolumeSource{Path: "/etc/localtime"}}},
 			{Name: "dockercfg", VolumeSource: v1.VolumeSource{HostPath: &v1.HostPathVolumeSource{Path: "/etc/sysconfig/docker"}}},
@@ -288,9 +350,36 @@ func newFluentdPodSpec(logging *logging.ClusterLogging, elasticsearchAppName str
 			{Name: "filebufferstorage", VolumeSource: v1.VolumeSource{HostPath: &v1.HostPathVolumeSource{Path: "/var/lib/fluentd"}}},
 			{Name: metricsVolumeName, VolumeSource: v1.VolumeSource{Secret: &v1.SecretVolumeSource{SecretName: "fluentd-metrics"}}},
 		},
-		logging.Spec.Collection.Logs.FluentdSpec.NodeSelector,
+		collectionSpec.Logs.FluentdSpec.NodeSelector,
 		tolerations,
 	)
+	for _, target := range pipelineSpec.Outputs {
+		if target.Secret != nil {
+			fluentdPodSpec.Volumes = append(fluentdPodSpec.Volumes, v1.Volume{Name: target.Name, VolumeSource: v1.VolumeSource{Secret: &v1.SecretVolumeSource{SecretName: target.Secret.Name}}})
+		}
+	}
+
+	if addTrustedCAVolume {
+		optional := true
+		fluentdPodSpec.Volumes = append(fluentdPodSpec.Volumes,
+			v1.Volume{
+				Name: constants.FluentdTrustedCAName,
+				VolumeSource: v1.VolumeSource{
+					ConfigMap: &v1.ConfigMapVolumeSource{
+						LocalObjectReference: v1.LocalObjectReference{
+							Name: constants.FluentdTrustedCAName,
+						},
+						Optional: &optional,
+						Items: []v1.KeyToPath{
+							{
+								Key:  constants.TrustedCABundleKey,
+								Path: constants.TrustedCABundleMountFile,
+							},
+						},
+					},
+				},
+			})
+	}
 
 	fluentdPodSpec.PriorityClassName = clusterLoggingPriorityClassName
 	// Shorten the termination grace period from the default 30 sec to 10 sec.
@@ -299,13 +388,31 @@ func newFluentdPodSpec(logging *logging.ClusterLogging, elasticsearchAppName str
 	return fluentdPodSpec
 }
 
-func (clusterRequest *ClusterLoggingRequest) createOrUpdateFluentdDaemonset() (err error) {
+func (clusterRequest *ClusterLoggingRequest) createOrUpdateFluentdDaemonset(pipelineConfHash string, proxyConfig *configv1.Proxy) (err error) {
 
 	cluster := clusterRequest.cluster
 
-	fluentdPodSpec := newFluentdPodSpec(cluster, "elasticsearch", "elasticsearch")
+	fluentdTrustBundle := &v1.ConfigMap{}
+	if proxyConfig != nil {
+		// Create or update cluster proxy trusted CA bundle.
+		err = clusterRequest.createOrUpdateTrustedCABundleConfigMap(constants.FluentdTrustedCAName)
+		if err != nil {
+			return
+		}
+
+		// fluentd-trusted-ca-bundle
+		fluentdTrustBundleName := types.NamespacedName{Name: constants.FluentdTrustedCAName, Namespace: constants.OpenshiftNS}
+		if err := clusterRequest.client.Get(context.TODO(), fluentdTrustBundleName, fluentdTrustBundle); err != nil {
+			if !errors.IsNotFound(err) {
+				return err
+			}
+		}
+	}
+
+	fluentdPodSpec := newFluentdPodSpec(cluster, "elasticsearch", "elasticsearch", proxyConfig, fluentdTrustBundle, clusterRequest.ForwardingSpec)
 
 	fluentdDaemonset := NewDaemonSet("fluentd", cluster.Namespace, "fluentd", "fluentd", fluentdPodSpec)
+	fluentdDaemonset.Spec.Template.Spec.Containers[0].Env = updateEnvVar(v1.EnvVar{Name: "FLUENT_CONF_HASH", Value: pipelineConfHash}, fluentdDaemonset.Spec.Template.Spec.Containers[0].Env)
 
 	uid := getServiceAccountLogCollectorUID()
 	if len(uid) == 0 {
@@ -323,7 +430,7 @@ func (clusterRequest *ClusterLoggingRequest) createOrUpdateFluentdDaemonset() (e
 
 	if clusterRequest.isManaged() {
 		retryErr := retry.RetryOnConflict(retry.DefaultRetry, func() error {
-			return clusterRequest.updateFluentdDaemonsetIfRequired(fluentdDaemonset)
+			return clusterRequest.updateFluentdDaemonsetIfRequired(fluentdDaemonset, fluentdTrustBundle)
 		})
 		if retryErr != nil {
 			return retryErr
@@ -333,8 +440,9 @@ func (clusterRequest *ClusterLoggingRequest) createOrUpdateFluentdDaemonset() (e
 	return nil
 }
 
-func (clusterRequest *ClusterLoggingRequest) updateFluentdDaemonsetIfRequired(desired *apps.DaemonSet) (err error) {
-	current := desired.DeepCopy()
+func (clusterRequest *ClusterLoggingRequest) updateFluentdDaemonsetIfRequired(desired *apps.DaemonSet, trustedCABundleCM *v1.ConfigMap) (err error) {
+	logger.DebugObject("desired fluent update: %v", desired)
+	current := &apps.DaemonSet{}
 
 	if err = clusterRequest.Get(desired.Name, current); err != nil {
 		if errors.IsNotFound(err) {
@@ -348,10 +456,21 @@ func (clusterRequest *ClusterLoggingRequest) updateFluentdDaemonsetIfRequired(de
 	flushBuffer := isBufferFlushRequired(current, desired)
 	desired, different := isDaemonsetDifferent(current, desired)
 
-	if different {
+	// Check trustedCA certs have been updated or not by comparing the hash values in annotation.
+	newTrustedCAHashedValue := calcTrustedCAHashValue(trustedCABundleCM)
+	trustedCAHashedValue, _ := current.Spec.Template.ObjectMeta.Annotations[constants.TrustedCABundleHashName]
+	if trustedCAHashedValue != newTrustedCAHashedValue {
+		different = true
+		if desired.Spec.Template.ObjectMeta.Annotations == nil {
+			desired.Spec.Template.ObjectMeta.Annotations = make(map[string]string)
+		}
+		desired.Spec.Template.ObjectMeta.Annotations[constants.TrustedCABundleHashName] = newTrustedCAHashedValue
+	}
 
+	if different {
+		current.Spec = desired.Spec
 		if flushBuffer {
-			current.Spec.Template.Spec.Containers[0].Env = append(current.Spec.Template.Spec.Containers[0].Env, v1.EnvVar{Name: "FLUSH_AT_SHUTDOWN", Value: "True"})
+			current.Spec.Template.Spec.Containers[0].Env = updateEnvVar(v1.EnvVar{Name: "FLUSH_AT_SHUTDOWN", Value: "True"}, current.Spec.Template.Spec.Containers[0].Env)
 			if err = clusterRequest.Update(current); err != nil {
 				logrus.Debugf("Failed to prepare Fluentd daemonset to flush its buffers: %v", err)
 				return err
@@ -362,7 +481,7 @@ func (clusterRequest *ClusterLoggingRequest) updateFluentdDaemonsetIfRequired(de
 				return fmt.Errorf("Timed out waiting for Fluentd to be ready")
 			}
 		}
-
+		logger.DebugObject("updating fluentd to: %v", current)
 		if err = clusterRequest.Update(desired); err != nil {
 			return err
 		}
@@ -370,3 +489,18 @@ func (clusterRequest *ClusterLoggingRequest) updateFluentdDaemonsetIfRequired(de
 
 	return nil
 }
+
+//updateEnvar adds the value to the list or replaces it if it already existing
+func updateEnvVar(value v1.EnvVar, values []v1.EnvVar) []v1.EnvVar {
+	found := false
+	for i, envvar := range values {
+		if envvar.Name == value.Name {
+			values[i] = value
+			found = true
+		}
+	}
+	if !found {
+		values = append(values, value)
+	}
+	return values
+}
